#  -*- coding:utf-8 -*-
from ctypes import *


class OrderUpdateData(object):
    def __init__(self, datac):
        self.acctType = datac.acctType.decode()         # 账户类型
        self.acct = datac.acct.decode()                 # 账户名称
        self.catsAcct = datac.catsAcct.decode()
        self.symbol = datac.symbol.decode()             # 合约代码
        self.orderQty = datac.orderQty                  # 报单总数量
        self.orderPrice = datac.orderPrice              # 报单价格
        self.tradeSide = datac.tradeSide.decode()       # 报单方向
        self.orderType = datac.orderType.decode()       # 报单类型
        self.orderParam = datac.orderParam.decode('utf-8', 'ignore')        # 报单参数
        self.orderNo = datac.orderNo.decode()

        self.corrType = datac.corrType.decode('utf-8', 'ignore')
        self.corrID = datac.corrID.decode()                                 # 内部ID; 算法单子单中，等于AlgoInstanceId
        self.filledQty = datac.filledQty                                    # 成交数量
        self.avgPrice = datac.avgPrice                                      # 成交均价
        self.cancelQty = datac.cancelQty                                    # 取消数量
        self.lastTime = datac.lastTime.decode('utf-8', 'ignore')            # 最后更新时间
        self.status = datac.status                                          # 报单状态
        self.orderTime = datac.orderTime.decode()                           # 委托时间
        self.orderDate = datac.orderDate.decode()                           # 委托日期
        self.text = datac.text.decode()                                     # 扩展信息

        self.remarks = datac.remarks.decode()
        self.clOrderNo = datac.clOrderNo.decode()                           # 下单时传入的客户端单号
        self.isHistPub = datac.isHistPub                                    # 是否为历史推送数据
        self.isHistPubFinished = datac.isHistPubFinished                    # 是否为历史推送数据的最后一笔记录。当该值为True时，前面的字段均为空，并且需要判断hisPubErrCode
        self.histPubErrCode = datac.histPubErrCode.decode()                 # 历史推送数据错误码。只有当isHistoryPubFinished为True时有用。
        self.subReqId = datac.subReqId.decode()
        self.localTs = datac.localTs

    def toString(self):
        return "OrderUpdateData(symbol=" + self.symbol + ", orderQty=" + str(self.orderQty) + ", orderPrice=" \
               + str(self.orderPrice) + ", tradeSide=" + self.tradeSide + ", orderType=" + self.orderType \
               + ", orderParam=" + self.orderParam + ", orderNo=" + self.orderNo + ", corrType=" + self.corrType \
               + ", corrID=" + self.corrID + ", filledQty=" + str(self.filledQty) + ", avgPrice=" + str(self.avgPrice) \
               + ", cancelQty= " + str(self.cancelQty) + ", lastTime=" + self.lastTime + ", status= " + str(self.status) \
               + ", orderTime= " + self.orderTime + ", orderDate=" + self.orderDate + ", text=" + self.text \
               + ", remarks=" + self.remarks + ", acctType=" + self.acctType + ", acct=" + self.acct \
               + ", catsAcct=" + self.catsAcct + ")"


class TransData(object):
    def __init__(self, datac):
        self.code = datac.code.decode()                         # 股票代码，带后缀，eg:600000.SH
        self.name = datac.name.decode()                         # 股票名称
        self.function_code = datac.function_code.decode()       # 成交代码，深市：'C', '0'
        self.order_kind = datac.order_kind.decode()             # 委托代码
        self.bs_flag = datac.bs_flag.decode()                   # BS标志, 买卖方向(买：'B', 卖：'S', 不明：' ')
        self.datetime = datac.datetime.decode()                 # 日期时间, eg:2019-01-03 09:30:23.310
        self.trading_date = datac.trading_date                  # 日期(交易日 YYYYMMDD)，eg:20190103，A股类标的同date
        self.date = datac.date                                  # 日期(自然日 YYYYMMDD)，eg:20190103
        self.time = datac.time                                  # 时间(HHMMSSmmm)，eg:93023310
        self.trade_price = datac.trade_price                    # 成交价格
        self.trade_volume = datac.trade_volume                  # 成交数量
        self.ask_order = datac.ask_order                        # 叫卖序号，eg:103906
        self.bid_order = datac.bid_order                        # 叫买序号，eg:60426

    def toString(self, n=2):
        if n < 0:
            n = 2
        n_format = f".{n}f"
        return f"TransData(code={self.code}, name={self.name}, function_code={self.function_code}, order_kind=" \
               f"{self.order_kind}, bs_flag={self.bs_flag}, datetime={self.datetime}, trading_date={self.trading_date}, " \
               f"date={self.date}, time={str(self.time)}, trade_price={self.trade_price:{n_format}}, trade_volume=" \
               f"{self.trade_volume}, ask_order={self.ask_order}, bid_order={self.bid_order})"


class TradeData(object):
    def __init__(self, datac):
        self.acctType = datac.acctType.decode()             # 账户类型
        self.acct = datac.acct.decode()                     # 账户名称
        self.symbol = datac.symbol.decode()                 # 商品代码
        self.tradeSide = datac.tradeSide.decode()           # 买卖方向
        self.orderNo = datac.orderNo.decode()               # 订单编号
        self.tradeNo = datac.tradeNo.decode()               # 成交编号
        self.quantity = datac.quantity                      # 成交数量
        self.price = datac.price                            # 成交价格
        self.time = datac.time.decode()                     # 成交时间 (HHmmss)
        self.tradeType = datac.tradeType.decode()           # 成交类型
        self.balance = datac.balance                        # 成交金额
        self.isHistPub = datac.isHistPub                    # 是否为历史推送数据
        self.isHistPubFinished = datac.isHistPubFinished    # 是否为历史推送数据的最后一笔记录。当该值为True时，前面的字段均为空，并且需要判断hisPubErrCode
        self.hisPubErrCode = datac.hisPubErrCode.decode()   # 历史推送数据错误码。只有当isHistoryPubFinished为True时有用。
        self.subReqId = datac.subReqId.decode()             # subscribe request id;
        self.localTs = datac.localTs                        # 本地时间戳(ms)。用于超时老化

    def toString(self):
        return "TradeData(symbol=" + self.symbol + ", tradeSide=" + self.tradeSide + ", orderNo=" + self.orderNo \
               + ", tradeNo=" + self.tradeNo + ", quantity=" + str(self.quantity) + ", price=" + str(self.price) \
               + ", time=" + self.time + ", tradeType=" + self.tradeType + ", balance=" + str(self.balance) \
               + ", acctType= " + self.acctType + ", acct=" + self.acct + ")"


class TickStruct(object):
    def __init__(self, datac):
        self.exchg = datac.exchg.decode()
        self.code = datac.code.decode()
        self.stock_name = datac.stock_name.decode()
        self.price = datac.price                        # 最新价
        self.open = datac.open                          # 开盘价
        self.high = datac.high                          # 最高价
        self.low = datac.low                            # 最低价
        self.settle_price = datac.settle_price          # 回测：结算价；实盘：对于期货表示今结算价, 对于股票表示总的买入量
        self.upper_limit = datac.upper_limit            # 涨停价
        self.lower_limit = datac.lower_limit            # 跌停价
        self.total_volume = datac.total_volume          # 总成交量
        self.volume = datac.volume                      # 成交量 现量，本次行情与上一次行情之间的交易量
        self.total_turnover = datac.total_turnover      # 总成交额
        self.turn_over = datac.turn_over                # 成交额
        self.open_interest = datac.open_interest        # 回测：总持；实盘：对期货而言表示总持，对股票而言，表示总卖出量
        self.diff_interest = datac.diff_interest        # 增仓
        self.trading_date = datac.trading_date          # 交易日
        self.action_date = datac.action_date            # 自然日期
        self.action_time = datac.action_time            # 发生时间,精确到毫秒
        self.pre_close = datac.pre_close                # 昨收价
        self.pre_settle = datac.pre_settle              # 回测：昨结算；实盘：对于期货表示昨结算价，对于股票表示总的卖出量
        self.pre_interest = datac.pre_interest          # 回测：上日总持；实盘：对期货而言表示上日总持，对股票而言，表示总卖出量
        self.bid_prices = datac.bid_prices              # 委买价格
        self.ask_prices = datac.ask_prices              # 委卖价格
        self.bid_qty = datac.bid_qty                    # 委买量
        self.ask_qty = datac.ask_qty                    # 委卖量

        # BEGIN : the following fields exist in real trading
        self.stop_mark = datac.stop_mark                # 是否停牌，1表示停牌
        self.future_code = datac.future_code.decode()
        self.ncjbs = datac.ncjbs
        self.close_price = datac.close_price
        self.pe_ratio1 = datac.pe_ratio1                # 深交所发布，行情市盈率1
        self.pe_ratio2 = datac.pe_ratio2                # 深交所发布，行情市盈率2
        self.cur_delta = datac.cur_delta                # 行情期货，昨虚实度
        self.pre_delta = datac.pre_delta                # 行情期货，今虚实度
        self.yieldToMaturity = datac.yieldToMaturity    # 到期收益率
        self.IOPV = datac.IOPV                          # IOPV 预估值
        self.accer = datac.accer                        # 涨速
        self.weighted_avg_bid_price = datac.weighted_avg_bid_price      # 加权平均委买价格
        self.weighted_avg_ask_price = datac.weighted_avg_ask_price      # 加权平均委卖价格
        # END : the following fields exist in real trading

    def toString(self, n=2):
        if n < 0:
            n = 2
        n_format = f".{n}f"
        return f"code={self.code}, tick=CATSBTTickStruct(action_date={self.action_date}, action_time={self.action_time}, " \
               f"price={self.price:{n_format}}, bid_prices[0]={self.bid_prices[0]:{n_format}}, " \
               f"ask_prices[0]={self.ask_prices[0]:{n_format}}, bid_qty[0]={self.bid_qty[0]}, ask_qty[0]={self.ask_qty[0]}, " \
               f"open={self.open:{n_format}}, high={self.high:{n_format}}, low={self.low:{n_format}}, " \
               f"turn_over={self.turn_over:{n_format}})"


class BarStruct(object):
    def __init__(self, datac):
        self.code = datac.code.decode()
        self.name = datac.name.decode()
        self.date = datac.date                          #  日期
        self.time = datac.time                          #  时间
        self.open = datac.open                          #  开盘价
        self.high = datac.high                          #  最高价
        self.low = datac.low                            #  最低价
        self.close = datac.close                        #  收盘价
        self.pre_close = datac.pre_close                #  昨收价

        self.turn_over = datac.turn_over                #  成交额
        self.volume = datac.volume                      #  成交量
        self.match_items = datac.match_items            #  成交笔数
        self.interest = datac.interest                  #  IOPV(基金)、利息(债券)

        self.settle = datac.settle                      #  结算
        self.hold = datac.hold                          #  总持
        self.add = datac.add                            #  增仓

    def toString(self, n=2):
        if n < 0:
            n = 2
        n_format = f".{n}f"
        return f"code={self.code}, bar=CATSBTBarStruct(date={self.date}, time={self.time}, open={self.open:{n_format}}, " \
               f"high={self.high:{n_format}}, low={self.low:{n_format}}, close={self.close:{n_format}}, " \
               f"pre_close={self.pre_close:{n_format}}, turn_over={self.turn_over:{n_format}}, " \
               f"volume={self.volume:{n_format}}, match_items={self.match_items}, interest={self.interest:{n_format}}, " \
               f"settle={self.settle:{n_format}}, hold={self.hold}, pre_close={self.pre_close:{n_format}})"


class OrderRequest(object):
    def __init__(self):
        self.acctType = ""
        self.acct = ""
        self.symbol = ""
        self.tradeSide = ""
        self.priceType = ""
        self.price = ""
        self.qty = 0
        self.orderParam = ""
        self.remarks = ""
        self.clOrderNo = ""

    def tostring(self):
        return "acctType=" + self.acctType + ",acct=" + self.acct + ",symbol=" + self.symbol + ",tradeSide=" + self.tradeSide + ",priceType=" + self.priceType \
               + ",price=" + self.price + ",qty=" + self.qty + ",orderParam=" + self.orderParam + ",remarks=" + self.remarks + ",clOrderNo=" + self.clOrderNo


class Fund(object):
    def __init__(self):
        self.currentBalance = 0.0  #  当前余额（期货结算准备金）
        self.enabledBalance = 0.0  #  可用余额
        self.moneyType = ""  #  货币类型
        self.fundAcct = ""  #  资金账户
        self.beginBalance = 0.0  #  期初余额
        self.fetchBalance = 0.0  #  可取金额
        self.frozenBalance = 0.0  #  冻结金额
        self.unfrozenBalace = 0.0  #  解冻金额
        self.deposit = 0.0  #  入金金额
        self.withdraw = 0.0  #  出金金额

        self.frozenMargin = 0.0  #  冻结保证金
        self.frozenCash = 0.0  #  冻结的资金
        self.currMargin = 0.0  #  当前保证金总额
        self.closeProfit = 0.0  #  平仓盈亏
        self.positionProfit = 0.0  #  持仓盈亏
        self.preBalance = 0.0  #  上次结算准备金
        self.preMargin = 0.0  #  上次占用的保证金
        self.preDeposit = 0.0  #  上次存款额
        self.preCredit = 0.0  #  上次信用额度
        self.preMortgage = 0.0  #  上次质押金额

        self.mortgage = 0.0  #  质押金额
        self.credit = 0.0  #  信用额度
        self.frozenCommission = 0.0  #  冻结手续费
        self.commission = 0.0  #  手续费
        self.dynamicEquity = 0.0  #  动态权益
        self.optionCloseProfile = 0.0  #  期权平仓盈亏
        self.enableBailBalance = 0.0  #  可用保证金
        self.usedBailBalance = 0.0  #  已用保证金
        self.usedPurBalance = 0.0  #  已用买入额度
        self.enablePurBalace = 0.0  #  可用买入额度

        self.netBuyAmount = 0.0  #  净买入额

        self.withdrawnRate = 0.0
        self.filledRate = 0.0
        self.rejectedRate = 0.0

        self.totalProfit = 0.0
        self.totalDynprofit = 0.0
        self.totalFees = 0.0

    def change(self, fundc):
        self.currentBalance = fundc.currentBalance
        self.enabledBalance = fundc.enabledBalance
        self.moneyType = fundc.moneyType.decode()
        self.fundAcct = fundc.fundAcct.decode()
        self.beginBalance = fundc.beginBalance
        self.fetchBalance = fundc.fetchBalance
        self.frozenBalance = fundc.frozenBalance
        self.unfrozenBalace = fundc.unfrozenBalace
        self.deposit = fundc.deposit
        self.withdraw = fundc.withdraw

        self.frozenMargin = fundc.frozenMargin
        self.frozenCash = fundc.frozenCash
        self.currMargin = fundc.currMargin
        self.closeProfit = fundc.closeProfit
        self.positionProfit = fundc.positionProfit
        self.preBalance = fundc.preBalance
        self.preMargin = fundc.preMargin
        self.preDeposit = fundc.preDeposit
        self.preCredit = fundc.preCredit
        self.preMortgage = fundc.preMortgage

        self.mortgage = fundc.mortgage
        self.credit = fundc.credit
        self.frozenCommission = fundc.frozenCommission
        self.commission = fundc.commission
        self.dynamicEquity = fundc.dynamicEquity
        self.optionCloseProfile = fundc.optionCloseProfile
        self.enableBailBalance = fundc.enableBailBalance
        self.usedBailBalance = fundc.usedBailBalance
        self.usedPurBalance = fundc.usedPurBalance
        self.enablePurBalace = fundc.enablePurBalace

        self.netBuyAmount = fundc.netBuyAmount
        self.withdrawnRate = fundc.withdrawnRate
        self.filledRate = fundc.filledRate
        self.rejectedRate = fundc.rejectedRate

        self.totalProfit = fundc.totalProfit
        self.totalDynprofit = fundc.totalDynprofit
        self.totalFees = fundc.totalFees

    def toString(self):
        return "Fund(fundAcct=" + str(self.fundAcct) + ",currentBalance=" + str(self.currentBalance) + \
               ",enabledBalance=" + str(self.enabledBalance) + ",frozenBalance=" + str(self.frozenBalance) + \
               ",totalFees=" + str(self.totalFees) + "beginBalance=" + str(self.beginBalance) + ")"


class Position(object):
    def __init__(self):
        self.symbol = ""
        self.currentQty = 0         # 当前数量
        self.enabledQty = 0         # 可用数量
        self.incomeBalance = 0      # 盈亏金额
        self.costPrice = 0          # 成本价  期权为持仓成本（累积买+当日买-累积卖-当日卖）
        self.lastPrice = 0          # 最新价
        self.marketValue = 0        # 参考市值
        self.stockName = ""         # 证券简称
        self.frozenQty = 0          # 冻结数量
        self.unfrozenQty = 0        # 解冻数量

        self.beginQty = 0           # 期初数量
        self.realBuyQty = 0         # 当天买入成交数量
        self.realSellQty = 0        # 当天卖出成交数量
        self.direction = ""         # 持仓多空方向：1-净;2-多头;3-空头  期权持仓类别,"0":权利方|"1":义务方|"2":备兑方
        self.positionDate = ""      # 持仓日期类型：1-今日持仓;2-历史持仓
        self.hedgeFlag = ""         # 投机套保标志：1-投机;2-套利;3-套保
        self.ydPosition = 0         # 上日持仓
        self.longFrozen = 0         # 多头冻结
        self.shortFrozen = 0        # 空头冻结
        self.useMargin = 0          # 占用的保证金

        self.positionCost = 0       # 持仓成本
        self.positionProfit = 0     # 持仓盈亏
        self.closeProfit = 0        # 平仓盈亏
        self.openVolume = 0         # 开仓数量
        self.openAmount = 0         # 开仓金额
        self.closeVolume = 0        # 平仓数量
        self.closeAmount = 0        # 平仓金额
        self.todayPosition = 0      # 今日持仓
        self.closeProfitByDate = 0      # 逐日盯市平仓盈亏
        self.closeProfitByTrade = 0     # 逐笔对冲平仓盈亏

        self.currentQtyCache_C = 0      # 客户端缓存当前持仓
        self.enabledQtyCache_C = 0      # 客户端缓存当前持仓可用数
        self.todayPositionCache_C = 0   # 客户端缓存今日持仓
        self.currentQtyCache_S = 0      # 服务端缓存当前持仓
        self.enabledQtyCache_S = 0      # 服务端缓存当前持仓可用数
        self.todayPositionCache_S = 0   # 服务端缓存今日持仓

        self.holdQty = 0                # 持有数量（包括委托未成交部分）
        self.realOpenQty = 0            # 今开仓持仓量
        self.realDropQty = 0            # 今平仓持仓量
        self.entrustDropQty = 0         # 今平仓委托量
        self.openAvgPrice = 0           # 开仓均价
        self.buyAvgQty = 0              # 买入均价
        self.avgIncomeBalance = 0       # 实现盈亏
        self.exerciseIncome = 0         # 行权盈亏金额
        self.dutyUsedBail = 0           # 义务仓占用保证金

    def change(self, positionc):
        self.symbol = positionc.symbol.decode()  # 
        self.currentQty = positionc.currentQty  #  当前数量
        self.enabledQty = positionc.enabledQty  #  可用数量
        self.incomeBalance = positionc.incomeBalance  #  盈亏金额
        self.costPrice = positionc.costPrice  #  成本价  期权为持仓成本（累积买+当日买-累积卖-当日卖）
        self.lastPrice = positionc.lastPrice  #  最新价
        self.marketValue = positionc.marketValue  #  参考市值
        self.stockName = positionc.stockName.decode()  #  证券简称
        self.frozenQty = positionc.frozenQty  #  冻结数量
        self.unfrozenQty = positionc.unfrozenQty  #  解冻数量

        self.beginQty = positionc.beginQty  #  期初数量
        self.realBuyQty = positionc.realBuyQty  #  当天买入成交数量
        self.realSellQty = positionc.realSellQty  #  当天卖出成交数量
        self.direction = positionc.direction.decode()  #  持仓多空方向：1-净;2-多头;3-空头  期权持仓类别,"0":权利方|"1":义务方|"2":备兑方
        self.positionDate = positionc.positionDate.decode()  #  持仓日期类型：1-今日持仓;2-历史持仓
        self.hedgeFlag = positionc.hedgeFlag.decode()  #  投机套保标志：1-投机;2-套利;3-套保
        self.ydPosition = positionc.ydPosition  #  上日持仓
        self.longFrozen = positionc.longFrozen  #  多头冻结
        self.shortFrozen = positionc.shortFrozen  #  空头冻结
        self.useMargin = positionc.useMargin  #  占用的保证金

        self.positionCost = positionc.positionCost  #  持仓成本
        self.positionProfit = positionc.positionProfit  #  持仓盈亏
        self.closeProfit = positionc.closeProfit  #  平仓盈亏
        self.openVolume = positionc.openVolume  #  开仓数量
        self.openAmount = positionc.openAmount  #  开仓金额
        self.closeVolume = positionc.closeVolume  #  平仓数量
        self.closeAmount = positionc.closeAmount  #  平仓金额
        self.todayPosition = positionc.todayPosition  #  今日持仓
        self.closeProfitByDate = positionc.closeProfitByDate  #  逐日盯市平仓盈亏
        self.closeProfitByTrade = positionc.closeProfitByTrade  #  逐笔对冲平仓盈亏

        self.currentQtyCache_C = positionc.currentQtyCache_C
        self.enabledQtyCache_C = positionc.enabledQtyCache_C
        self.todayPositionCache_C = positionc.todayPositionCache_C
        self.currentQtyCache_S = positionc.currentQtyCache_S
        self.enabledQtyCache_S = positionc.enabledQtyCache_S
        self.todayPositionCache_S = positionc.todayPositionCache_S

        self.holdQty = positionc.holdQty  #  持有数量（包括委托未成交部分）
        self.realOpenQty = positionc.realOpenQty  #  今开仓持仓量
        self.realDropQty = positionc.realDropQty  #  今平仓持仓量
        self.entrustDropQty = positionc.entrustDropQty  #  今平仓委托量
        self.openAvgPrice = positionc.openAvgPrice  #  开仓均价
        self.buyAvgQty = positionc.buyAvgQty  #  买入均价
        self.avgIncomeBalance = positionc.avgIncomeBalance  #  实现盈亏
        self.exerciseIncome = positionc.exerciseIncome  #  行权盈亏金额
        self.dutyUsedBail = positionc.dutyUsedBail  #  义务仓占用保证金

    def toString(self):
        return "Position(symbol=" + self.symbol + ",stockName=" + self.stockName + ',currentQty=' \
               + str(self.currentQty) + ',enabledQty=' + str(self.enabledQty) + ',frozenQty=' + str(self.frozenQty) \
               + ',costPrice=' + str(self.costPrice) + ',lastPrice=' + str(self.lastPrice) + ',marketValue=' \
               + str(self.marketValue) + ',incomeBalance=' + str(self.incomeBalance)


class DataField(object):
    def __init__(self):
        self.name = b""
        self.value = b""
        self.type = 0


class DataRow(object):
    def __init__(self):
        self.ncol = 0
        self.fields = []


class DataTable(object):
    def __init__(self):
        self.nrow = 0
        self.ncol = 0
        self.rows = []

