﻿#pragma once

#include <iostream>
#include <sstream>
#include <stdlib.h>
#include <vector>
#include <deque>
#include <string.h>
#include <chrono>
#include "CATSBTMarcos.h"
#include "CATSBTObject.hpp"

#pragma pack(push, 1)

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

using namespace std;

NS_OTP_BEGIN


typedef void(*INTERVAL_CALLBACK_FUNC)(void *arg);

enum BackTestType
{
	TICK = 0, //Tick
	MINUTE,   //分钟线
	DAILY	  //日线
};

typedef enum OrderUpdateStatus_T
{
	NEW = 0,
	PARTIAL_FILLED = 1,
	FILLED = 2,
	PRARITAL_CANCELED = 3,
	CANCELED = 4,
	REJECTED = 5,
	UNKNOWN
}OrderUpdateStatus;

struct OrderUpdateData {
	std::string acctType;// 账户类型
	std::string acct;    // 账户名称
	std::string catsAcct; 
	std::string symbol;  // 合约代码
	int orderQty = 0; // 报单总数量
	float orderPrice;  // 报单价格
	std::string tradeSide; // 报单方向
	std::string orderType;  // 报单类型
	std::string orderParam;  // 报单参数
	std::string orderNo;  

	std::string corrType; // corrType
	std::string corrID;   // 内部ID; 算法单子单中，等于AlgoInstanceId

	int filledQty = 0;  // 成交数量
	float avgPrice;  // 成交均价
	int cancelQty = 0; // 取消数量

	std::string lastTime; //最后更新时间  格式为 hhmmss

	/*status 参数说明
	0 新单(未结)
	1 部分成交(未结)
	2 全成(已结)
	3 部分撤单(已结)
	4 全撤(已结)
	5 拒单(已结)
	*/
	OrderUpdateStatus status; // 报单状态

	std::string orderTime;  // 委托时间  格式为hhmmss
	std::string orderDate;  // 委托日期  格式yyyymmdd
	std::string text;		// 扩展信息。目前包含拒单原因
	std::string remarks;	// 
	std::string clOrderNo; //客户下单时的自己的信息

	//the following fields for real operation
	bool isHistPub = false;				// 是否为历史推送数据
	bool isHistPubFinished = false;		// 是否为历史推送数据的最后一笔记录。当该值为True时，前面的字段均为空，并且需要判断hisPubErrCode
	std::string histPubErrCode;		// 历史推送数据错误码。只有当isHistoryPubFinished为True时有用。
	
	std::string subReqId;		// 
	long long localTs = 0;		// 

	OrderUpdateData() {
		acctType = "";
		acct = "";
		catsAcct = "";
		symbol = "";
		orderQty = 0;
		orderPrice = 0;
		tradeSide = "";
		orderType = "";
		orderParam = "";
		orderNo = "";
		corrType = "";
		corrID = "";
		filledQty = 0;
		avgPrice = 0.0;
		cancelQty = 0;
		lastTime = "";
		status = OrderUpdateStatus::NEW; //new order
		orderTime = "";
		orderDate = "";
		text = "";
		remarks = "";
		clOrderNo = "";

		isHistPub = false;
		isHistPubFinished = false;
		histPubErrCode = "";
		subReqId = "";
		localTs = 0;
	}


	bool isFinished() const
	{
		return isFinished(status);
	}

	static bool isFinished(OrderUpdateStatus s)
	{
		if (s == OrderUpdateStatus::FILLED
			|| s == OrderUpdateStatus::PRARITAL_CANCELED
			|| s == OrderUpdateStatus::CANCELED
			|| s == OrderUpdateStatus::REJECTED) {
			return true;
		}
		return false;		
	}

	bool isAllFilled() const
	{
		return (orderQty == filledQty);
	}

	string toString() const {
		std::stringstream oss;
		oss << "OrderUpdateData("
			APPEND_MEMBER0(symbol)
			APPEND_MEMBER(orderQty)
			APPEND_MEMBER(orderPrice)
			APPEND_MEMBER(tradeSide)
			APPEND_MEMBER(orderType)
			APPEND_MEMBER(orderParam)
			APPEND_MEMBER(orderNo)
			APPEND_MEMBER(corrType)
			APPEND_MEMBER(corrID)
			APPEND_MEMBER(filledQty)
			APPEND_MEMBER(avgPrice)
			APPEND_MEMBER(cancelQty)
			APPEND_MEMBER(lastTime)
			APPEND_MEMBER(status)
			APPEND_MEMBER(orderTime)
			APPEND_MEMBER(orderDate)
			APPEND_MEMBER(text)
			APPEND_MEMBER(remarks)
			APPEND_MEMBER(clOrderNo)
			APPEND_MEMBER(acctType)
			APPEND_MEMBER(acct)
			APPEND_MEMBER(catsAcct)

			<< ")";
		return oss.str();
	}
};


struct TradeData {
	std::string acctType;// 账户类型
	std::string acct;    // 账户名称
	std::string symbol;  // 商品代码
	std::string tradeSide;  // 买卖方向
	std::string orderNo;  // 订单编号
	std::string tradeNo;   // 成交编号
	int quantity = 0; // 成交数量
	float price;  // 成交价格
	std::string time; // 成交时间 (HHmmss)
	std::string tradeType;  // 成交类型   
	double balance;  // 成交金额   

	//the following fields for real operation
	bool isHistPub = false;				// 是否为历史推送数据
	bool isHistPubFinished = false;		// 是否为历史推送数据的最后一笔记录。当该值为True时，前面的字段均为空，并且需要判断hisPubErrCode
	std::string histPubErrCode;	// 历史推送数据错误码。只有当isHistoryPubFinished为True时有用。

	std::string subReqId;		// subscribe request id;
	long long localTs = 0;		// 本地时间戳(ms)。用于超时老化

	TradeData() {
		acctType = "";
		acct = "";
		symbol = "";
		tradeSide = "";
		orderNo = "";
		tradeNo = "";
		time = "";
		tradeType = "";
		price = 0;
		balance = 0;
		isHistPub = false;
		isHistPubFinished = false;
		histPubErrCode = "";
		subReqId = "";
		localTs = 0;
	}

	string toString() const {
		std::stringstream oss;
		oss << "TradeData("
			APPEND_MEMBER0(symbol)
			APPEND_MEMBER(tradeSide)
			APPEND_MEMBER(orderNo)
			APPEND_MEMBER(tradeNo)
			APPEND_MEMBER(quantity)
			APPEND_MEMBER(price)
			APPEND_MEMBER(time)
			APPEND_MEMBER(tradeType)
			APPEND_MEMBER(balance)
			APPEND_MEMBER(acctType)
			APPEND_MEMBER(acct)

			<< ")";
		return oss.str();
	}
};


struct OrderRequest {
	std::string acctType;
	std::string  acct;
	std::string symbol;

	/*
	"1";  // 买入  股票
	"2";  // 卖出  股票

	"FA"; // 开多仓（开仓买入）  股指期货
	"FB"; // 开空仓（开仓卖出）  股指期货
	"FC"; // 平空仓（平仓买入）  股指期货
	"FD"; // 平多仓（平仓卖出）  股指期货

	*/
	std::string tradeSide;

	/* 
	上海 股票
	0	限价单
	U	市价单（最优五档即时成交剩余撤销）
	R	市价单（最优五档即时成交剩余转限价）
	深圳 股票、信用
	0	限价单
	U	市价单（最优五档即时成交剩余撤销）
	S	市价单（本方最优价格）
	T	市价单（即时成交剩余撤销）
	Q	市价单（对手方最优价格）
	V	市价单（全额成交或撤单）
	*/
	std::string priceType;


	float price;
	uint32_t qty;
	std::string orderParam;
	std::string reamrks; //暂时不用
	std::string clOrderNo; //下单时的参数，在 on_order回调函数中，通过OrderUpdate中的cltOrderNo返回

	OrderRequest() {
		acctType = "";
		acct = "";
		symbol = "";
		priceType = "0"; 
		orderParam = "";
		reamrks = "";
		clOrderNo = "";
	}
};



struct Fund
{
	double currentBalance;	// 当前余额（期货结算准备金）
	double enabledBalance;	// 可用余额
	char   moneyType[MAX_MONEY_TYPE_LENGTH];		// 货币类型

							/* 证券相关 */
	char   fundAcct[MAX_ACCT_LENGTH];	// 资金账户

	double beginBalance;    //期初余额
	double fetchBalance;    //可取金额
	double frozenBalance;   //冻结金额
	double unfrozenBalace;  //解冻金额

							/* 期货、期权相关 */
	double deposit;         //入金金额
	double withdraw;        //出金金额
	double frozenMargin;    //冻结保证金
	double frozenCash;      //冻结的资金
	double currMargin;      //当前保证金总额
	double closeProfit;     //平仓盈亏
	double positionProfit;  //持仓盈亏	

	double preBalance;		//上次结算准备金
	double preMargin;		//上次占用的保证金
	double preDeposit;		//上次存款额
	double preCredit;		//上次信用额度
	double preMortgage;		//上次质押金额
	double mortgage;		//质押金额
	double credit;			//信用额度
	double frozenCommission; //冻结手续费
	double commission;		//手续费
	double dynamicEquity;	//动态权益

							/* 期权相关 */
	double optionCloseProfile;//期权平仓盈亏

							  //个股期权
	double enableBailBalance;    //可用保证金 
	double usedBailBalance;  	 //已用保证金
	double usedPurBalance;   	 //已用买入额度
	double enablePurBalace;  	 //可用买入额度

	double netBuyAmount;		//净买入额

	double withdrawnRate;
	double filledRate;
	double rejectedRate;

								//回测
	double	totalProfit;		
	double	totalDynprofit;
	double	totalFees;

	Fund()
	{
		memset(this, 0, sizeof(Fund));
	}
	
	string toString() const {
		stringstream oss;
		oss << "Fund(fundAcct=" << fundAcct
			<< ",currentBalance=" << fixed << currentBalance
			<< ",enabledBalance=" << fixed << enabledBalance
			<< ",frozenBalance=" << fixed << frozenBalance
			<< ",totalFees=" << fixed << totalFees
			<< ",beginBalance=" << fixed << beginBalance
			<< ")";
		return oss.str();
	}
};

struct Position
{
	char symbol[MAX_INSTRUMENT_LENGTH];
	int currentQty;	// 当前数量
	int enabledQty;	// 可用数量
	double incomeBalance;	// 盈亏金额	

							/* 证券相关 */
	float costPrice;		// 成本价  期权为持仓成本（累积买+当日买-累积卖-当日卖）
	float lastPrice;		// 最新价
	double marketValue;		// 参考市值
	char  stockName[MAX_INSTRUMENT_NAME_LENGTH];		// 证券简称

	int frozenQty;   	//冻结数量
	int unfrozenQty; 	//解冻数量

	int beginQty;    	//期初数量
	int realBuyQty;  	//当天买入成交数量
	int realSellQty; 	//当天卖出成交数量

						/* 期货相关 */
	char direction[2];		 //持仓多空方向：1-净;2-多头;3-空头  期权持仓类别,"0":权利方|"1":义务方|"2":备兑方
	char positionDate[2];    //持仓日期类型：1-今日持仓;2-历史持仓
	char hedgeFlag[2];       //投机套保标志：1-投机;2-套利;3-套保
	int ydPosition;     //上日持仓
	int longFrozen;     //多头冻结
	int shortFrozen;    //空头冻结 
	double useMargin;        //占用的保证金
	double positionCost;     //持仓成本
	double positionProfit;	 //持仓盈亏
	double closeProfit;      //平仓盈亏

	int openVolume;			//开仓数量	
	double openAmount;       //开仓金额	
	int closeVolume;		//平仓数量 	
	double closeAmount;      //平仓金额
	int todayPosition;		//今日持仓

	double closeProfitByDate; //逐日盯市平仓盈亏
	double closeProfitByTrade;//逐笔对冲平仓盈亏

	int currentQtyCache_C;   // 客户端缓存当前持仓
	int enabledQtyCache_C;   // 客户端缓存当前持仓可用数
	int todayPositionCache_C;   // 客户端缓存今日持仓
	int currentQtyCache_S;   // 服务端缓存当前持仓
	int enabledQtyCache_S;   // 服务端缓存当前持仓可用数
	int todayPositionCache_S;   // 服务端缓存今日持仓

							 /* 期权相关 */
	double holdQty;  		// 持有数量（包括委托未成交部分）
	double realOpenQty;		// 今开仓持仓量
	double realDropQty;  	// 今平仓持仓量
	double entrustDropQty;	// 今平仓委托量	
	double openAvgPrice;		// 开仓均价
	double buyAvgQty;  		// 买入均价
	double avgIncomeBalance;// 实现盈亏
	double exerciseIncome;	// 行权盈亏金额
	double dutyUsedBail;		// 义务仓占用保证金

	Position()
	{
		memset(this, 0, sizeof(Position));
	}
	
	string toString() const {
		std::string symbolStr = symbol == NULL ? "" : symbol;
		std::string stockNameStr = stockName == NULL ? "" : stockName;
		std::stringstream oss;
		oss << "Position(symbol=" << symbolStr;
		oss << ",stockName=" << stockNameStr;
		oss << ","
			APPEND_MEMBER0(currentQty)
			APPEND_MEMBER(enabledQty)
			APPEND_MEMBER(frozenQty)
			APPEND_MEMBER(costPrice)
			APPEND_MEMBER(lastPrice)
			APPEND_MEMBER(marketValue)
			APPEND_MEMBER(incomeBalance)
			<< ")";
		return oss.str();
	}
};

enum CommodityType
{
	ASHARESTOCK = 0, //a-share
	ASHAREINDEXFUTURE //a-share index future 
};

struct CommodityInfo {
	int multiplier;
	float ticksize;	
	int lotsize;	
	int minorderqty;
	CommodityType commodity_type;
};

struct CATSBTBarStruct
{
public:
	CATSBTBarStruct()
	{
		memset(this, 0, sizeof(CATSBTBarStruct));
	}

	char		code[MAX_INSTRUMENT_LENGTH];
	char		name[MAX_INSTRUMENT_LENGTH];
	uint32_t	date;
	uint32_t	time;
	float		open;		//开
	float		high;		//高
	float		low;		//低
	float		close;		//收
	float		pre_close;	//前收

	double		turover;	//成交额
	uint64_t	volume;		//成交量
	uint64_t	match_items;//成交笔数
	float		interest;	//IOPV(基金)、利息(债券)

	float		settle;		//结算
	uint64_t	hold;	//总持
	int64_t		add;	//增仓

	int compare(uint32_t curr_date, uint32_t curr_time)
	{
		if (date < curr_date) return -1;
		else if (date > curr_date) return 1;
		else {
			if (time < curr_time) return -1;
			else if (time > curr_time) return 1;
			else return 0;
		}
	}

	string toString() const
	{
		std::stringstream oss;
		oss << "CATSBTBarStruct("
			APPEND_MEMBER0(code)
			APPEND_MEMBER(name)
			APPEND_MEMBER(date)
			APPEND_MEMBER(time)
			APPEND_MEMBER(open)
			APPEND_MEMBER(high)
			APPEND_MEMBER(low)
			APPEND_MEMBER(close)
			APPEND_MEMBER(pre_close)
			APPEND_MEMBER(turover)
			APPEND_MEMBER(volume)
			APPEND_MEMBER(match_items)
			APPEND_MEMBER(interest)
			APPEND_MEMBER(settle)
			APPEND_MEMBER(hold)
			APPEND_MEMBER(add)
			<< ")";
		return oss.str();
	}
};

class CATSBTBarData : public CATSBTObject
{
public:
	static inline CATSBTBarData* create(const char* code)
	{
		CATSBTBarData* pRet = new CATSBTBarData;
		strcpy(pRet->m_tsStruct.code, code);
		return pRet;
	}

	static inline CATSBTBarData* create(CATSBTBarStruct& barData)
	{
		CATSBTBarData* pRet = new CATSBTBarData;
		memcpy(&pRet->m_tsStruct, &barData, sizeof(CATSBTBarStruct));

		return pRet;
	}

	inline const char* code() const { return m_tsStruct.code; }
	inline const char* name() const { return m_tsStruct.name; }
	inline uint32_t date() const { return m_tsStruct.date; }
	inline uint32_t time() const { return m_tsStruct.time; }
	inline float open() const { return m_tsStruct.open; }
	inline float high() const { return m_tsStruct.high; }
	inline float low() const { return m_tsStruct.low; }
	inline float close() const { return m_tsStruct.close; }
	inline float pre_close() const { return m_tsStruct.pre_close; }
	inline double turover() const { return m_tsStruct.turover; }				//成交额
	inline uint64_t volume() const { return m_tsStruct.volume; }				//成交量
	inline uint64_t match_items() const { return m_tsStruct.match_items; }		//成交笔数
	inline float interest() const { return m_tsStruct.interest; }				//IOPV(基金)、利息(债券)
	inline float settle() const { return m_tsStruct.settle; }					//结算
	inline uint64_t hold() const { return m_tsStruct.hold; }					//总持
	inline uint64_t add() const { return m_tsStruct.add; }						//增仓
	

	inline CATSBTBarStruct& getBarStruct() { return m_tsStruct; }

	inline void	setCode(const char* code) { strcpy(m_tsStruct.code, code); }

	string  toString() const
	{
		std::stringstream oss;
		oss << "CATSBTBarData("
			APPEND_SUB_MEMBER0(m_tsStruct)
			<< ")";
		return oss.str();
	}

private:
	CATSBTBarStruct	m_tsStruct;
};

NS_OTP_END;

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif


#pragma pack(pop)
