﻿#pragma once

#include <iostream>
#include <sstream>
#include <stdlib.h>
#include <vector>
#include <deque>
#include <string.h>
#include <chrono>
#include "CATSBTMarcos.h"
#include "CATSBTObject.hpp"

#pragma pack(push, 1)

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

using namespace std;

NS_OTP_BEGIN

struct CATSBTTickStruct
{
	char		exchg[MAX_EXCHANGE_LENGTH];
	char		code[MAX_INSTRUMENT_LENGTH];
	char		stock_name[MAX_INSTRUMENT_LENGTH];

	float		price;				//最新价
	float		open;				//开盘价
	float		high;				//最高价
	float		low;				//最低价
	float		settle_price;		//回测：结算价; 实盘：对于期货表示今结算价, 对于股票表示总的买入量

	float		upper_limit;		//涨停价
	float		lower_limit;		//跌停价

	uint64_t	total_volume;		//总成交量
	uint64_t	volume;				//成交量 现量，本次行情与上一次行情之间的交易量
	double		total_turnover;		//总成交额
	double		turn_over;			//成交额
	int64_t		open_interest;		//回测：总持；实盘：对期货而言表示总持，对股票而言，表示总卖出量
	int32_t		diff_interest;		//增仓

	uint32_t	trading_date;		//交易日,如20140327
	uint32_t	action_date;		//自然日期,如20140327
	uint32_t	action_time;		//发生时间,精确到毫秒,如105932000

	float		pre_close;			//昨收价
	float		pre_settle;			//回测：昨结算；实盘：对于期货表示昨结算价，对于股票表示总的卖出量
	int64_t		pre_interest;		//回测：上日总持；实盘：对期货而言表示上日总持，对股票而言，表示总卖出量

	float		bid_prices[10];		//委买价格
	float		ask_prices[10];		//委卖价格
	uint32_t	bid_qty[10];		//委买量
	uint32_t	ask_qty[10];		//委卖量


	//BEGIN : the following fields exist in real trading
	int stop_mark;    // 是否停牌，1表示停牌
	char future_code[64];
	int ncjbs;	
	float close_price;

	float pe_ratio1;  // 深交所发布，行情市盈率1
	float pe_ratio2;  // 深交所发布，行情市盈率2
	float cur_delta;  // 行情期货，昨虚实度
	float pre_delta;  // 行情期货，今虚实度 
	int yieldToMaturity;  // 到期收益率

	float IOPV; // IOPV 预估值
	float accer;   //涨速
	float weighted_avg_bid_price;  //加权平均委买价格
	float weighted_avg_ask_price; //加权平均委卖价格
	//END : the following fields exist in real trading


	CATSBTTickStruct()
	{
		memset(this, 0, sizeof(CATSBTTickStruct));
	}

	int compare(uint32_t date, uint32_t time)
	{
		if (action_date < date) return -1;
		else if (action_date > date) return 1;
		else {
			if (action_time < time) return -1;
			else if (action_time > time) return 1;
			else return 0;
		}
	}
	int compare(CATSBTTickStruct tick)
	{
		if (action_date < tick.action_date) return -1;
		else if (action_date > tick.action_date) return 1;
		else {
			if (action_time < tick.action_time) return -1;
			else if (action_time > tick.action_time) return 1;
			else return 0;
		}
	}

	string toString() const
	{
		std::stringstream oss;
		oss << "CATSBTTickStruct("
			APPEND_MEMBER0(code)
			APPEND_MEMBER(stock_name)
			APPEND_MEMBER(price)
			APPEND_MEMBER(open)
			APPEND_MEMBER(high)
			APPEND_MEMBER(low)
			APPEND_MEMBER(settle_price)
			APPEND_MEMBER(upper_limit)
			APPEND_MEMBER(lower_limit)
			APPEND_MEMBER(total_volume)
			APPEND_MEMBER(volume)
			APPEND_MEMBER(total_turnover)
			APPEND_MEMBER(turn_over)
			APPEND_MEMBER(open_interest)
			APPEND_MEMBER(diff_interest)
			APPEND_MEMBER(trading_date)
			APPEND_MEMBER(action_date)
			APPEND_MEMBER(action_time)
			APPEND_MEMBER(pre_close)
			APPEND_MEMBER(pre_settle)
			APPEND_MEMBER(pre_interest)
			APPEND_MEMBER(bid_prices[0])
			APPEND_MEMBER(ask_prices[0])
			APPEND_MEMBER(bid_qty[0])
			APPEND_MEMBER(ask_qty[0])
			<< ")";
		return oss.str();
	}

};

struct CATSBTTransStruct
{
	char		code[MAX_INSTRUMENT_LENGTH];			//股票代码，带后缀，eg:600000.SH
	char		name[32];			//股票名称
	char		function_code[32];	//成交代码，深市：'C', '0'
	char		order_kind[32];		//委托代码
	char		bs_flag[8];			//BS标志, 买卖方向(买：'B', 卖：'S', 不明：' ')
	char		datetime[32];		//日期时间, eg:2019-01-03 09:30:23.310

	uint32_t		trading_date;			//日期(交易日 YYYYMMDD)，eg:20190103，A股类标的同date
	uint32_t		date;					//日期(自然日 YYYYMMDD)，eg:20190103
	long long		time;					//时间(HHMMSSmmm)，eg:93023310
	double			trade_price;			//成交价格
	uint32_t		trade_volume;			//成交数量
	uint32_t		ask_order;				//叫卖序号，eg:103906
	uint32_t		bid_order;				//叫买序号，eg:60426

	CATSBTTransStruct()
	{
		memset(this, 0, sizeof(CATSBTTransStruct));
	}

	int compare(CATSBTTransStruct trans)
	{
		if (date < trans.date) return -1;
		else if (date > trans.date) return 1;
		else {
			if (time < trans.time) return -1;
			else if (time > trans.time) return 1;
			else return 0;
		}
	}

	int compare(uint32_t curr_date, long long curr_time)
	{
		if (date < curr_date) return -1;
		else if (date > curr_date) return 1;
		else {
			if (time < curr_time) return -1;
			else if (time > curr_time) return 1;
			else return 0;
		}
	}

	string toString() const
	{
		std::stringstream oss;
		oss << "CATSBTTransStruct("
			APPEND_MEMBER0(date)
			APPEND_MEMBER(time)
			APPEND_MEMBER(trade_price)
			APPEND_MEMBER(trade_volume)
			APPEND_MEMBER(bs_flag)
			APPEND_MEMBER(function_code)
			APPEND_MEMBER(order_kind)
			<< ")";
		return oss.str();
	}
};

/*
*	Tick数据对象
*	内部封装WTSTickStruct
*	封装的主要目的是出于跨语言的考虑
*/
class CATSBTTickData : public CATSBTObject
{
public:
	/*
	*	创建一个tick数据对象
	*	@stdCode 合约代码
	*/
	static inline CATSBTTickData* create(const char* stdCode)
	{
		CATSBTTickData* pRet = new CATSBTTickData;
		strcpy(pRet->m_tickStruct.code, stdCode);

		return pRet;
	}


	/*
	*	根据tick结构体创建一个tick数据对象
	*	@tickData tick结构体
	*/
	static inline CATSBTTickData* create(CATSBTTickStruct& tickData)
	{
		CATSBTTickData* pRet = new CATSBTTickData;
		memcpy(&pRet->m_tickStruct, &tickData, sizeof(CATSBTTickStruct));

		return pRet;
	}

	inline void setCode(const char* code)
	{
		strcpy(m_tickStruct.code, code);
	}
	
	/*
	*	读取合约代码
	*/
	inline const char* code() const { return m_tickStruct.code; }

	/*
	*	读取市场代码
	*/
	inline const char*	exchg() const { return m_tickStruct.exchg; }

	/*
	*	读取最新价
	*/
	inline float	price() const { return m_tickStruct.price; }

	inline float	open() const { return m_tickStruct.open; }

	/*
	*	最高价
	*/
	inline float	high() const { return m_tickStruct.high; }

	/*
	*	最低价
	*/
	inline float	low() const { return m_tickStruct.low; }

	//昨收价,如果是期货则是昨结算
	inline float	preclose() const { return m_tickStruct.pre_close; }
	inline float	presettle() const { return m_tickStruct.pre_settle; }
	inline int32_t	preinterest() const { return m_tickStruct.pre_interest; }

	inline float	upperlimit() const { return m_tickStruct.upper_limit; }
	inline float	lowerlimit() const { return m_tickStruct.lower_limit; }
	//成交量
	inline uint64_t	totalvolume() const { return m_tickStruct.total_volume; }

	//成交量
	inline uint64_t	volume() const { return m_tickStruct.volume; }

	//结算价
	inline float	settlepx() const { return m_tickStruct.settle_price; }

	//总持
	inline uint32_t	openinterest() const { return m_tickStruct.open_interest; }

	inline int32_t	additional() const { return m_tickStruct.diff_interest; }

	//成交额
	inline double	totalturnover() const { return m_tickStruct.total_turnover; }

	//成交额
	inline double	turnover() const { return m_tickStruct.turn_over; }

	//交易日
	inline uint32_t	tradingdate() const { return m_tickStruct.trading_date; }

	//数据发生日期
	inline uint32_t	actiondate() const { return m_tickStruct.action_date; }

	//数据发生时间
	inline uint32_t	actiontime() const { return m_tickStruct.action_time; }


	/*
	*	读取指定档位的委买价
	*	@idx 0-9
	*/
	inline float		bidprice(int idx) const
	{
		if (idx < 0 || idx >= 10)
			return -1;

		return m_tickStruct.bid_prices[idx];
	}

	/*
	*	读取指定档位的委卖价
	*	@idx 0-9
	*/
	inline float		askprice(int idx) const
	{
		if (idx < 0 || idx >= 10)
			return -1;

		return m_tickStruct.ask_prices[idx];
	}

	/*
	*	读取指定档位的委买量
	*	@idx 0-9
	*/
	inline uint32_t	bidqty(int idx) const
	{
		if (idx < 0 || idx >= 10)
			return -1;

		return m_tickStruct.bid_qty[idx];
	}

	/*
	*	读取指定档位的委卖量
	*	@idx 0-9
	*/
	inline uint32_t	askqty(int idx) const
	{
		if (idx < 0 || idx >= 10)
			return -1;

		return m_tickStruct.ask_qty[idx];
	}

	/*
	*	返回tick结构体的引用
	*/
	inline CATSBTTickStruct&	getTickStruct() { return m_tickStruct; }

	
	string  toString() const
	{
		std::stringstream oss;
		oss << "CATSBTTickData("
			APPEND_SUB_MEMBER0(m_tickStruct)
			<< ")";
		return oss.str();
	}


private:
	CATSBTTickStruct	m_tickStruct;
	uint64_t		m_uLocalTime;	//本地时间

	CATSBTTickData() :m_uLocalTime(std::chrono::duration_cast<std::chrono::nanoseconds>(std::chrono::system_clock::now().time_since_epoch()).count()) {}
};


class CATSBTTransData : public CATSBTObject
{
public:
	static inline CATSBTTransData* create(const char* code)
	{
		CATSBTTransData* pRet = new CATSBTTransData;
		strcpy(pRet->m_tsStruct.code, code);
		return pRet;
	}

	static inline CATSBTTransData* create(CATSBTTransStruct& transData)
	{
		CATSBTTransData* pRet = new CATSBTTransData;
		memcpy(&pRet->m_tsStruct, &transData, sizeof(CATSBTTransStruct));

		return pRet;
	}
	
	inline const char* code() const { return m_tsStruct.code; }
	inline const char* name() const { return m_tsStruct.name; }
	inline const char* function_code() const { return m_tsStruct.function_code; }
	inline const char* order_kind() const { return m_tsStruct.order_kind; }
	inline const char* bs_flag() const { return m_tsStruct.bs_flag; }
	inline const char* datetime() const { return m_tsStruct.datetime; }
	inline uint32_t tradingdate() const { return m_tsStruct.trading_date; }
	inline uint32_t actiondate() const { return m_tsStruct.date; }
	inline uint32_t actiontime() const { return m_tsStruct.time; }
	inline double trade_price() const { return m_tsStruct.trade_price; }
	inline uint32_t trade_volume() const { return m_tsStruct.trade_volume; }
	inline uint32_t ask_order() const { return m_tsStruct.ask_order; }
	inline uint32_t bid_order() const { return m_tsStruct.bid_order; }

	inline CATSBTTransStruct& getTransStruct() { return m_tsStruct; }

	inline void		setCode(const char* code) { strcpy(m_tsStruct.code, code); }

	string  toString() const
	{
		std::stringstream oss;
		oss << "CATSBTTransData("
			APPEND_SUB_MEMBER0(m_tsStruct)
			<< ")";
		return oss.str();
	}

private:
	CATSBTTransStruct	m_tsStruct;
};


//////////////////////////////////////////////////////////////////////////
/*
*	@brief Tick数据切片,从连续的tick缓存中做的切片
*	@details 切片并没有真实的复制内存,而只是取了开始和结尾的下标
*	这样使用虽然更快,但是使用场景要非常小心,因为他依赖于基础数据对象
*/
class CATSBTTickSlice : public CATSBTObject
{
private:
	char			m_strCode[MAX_INSTRUMENT_LENGTH];
	CATSBTTickStruct*	m_ptrBegin;
	uint32_t		m_uCount;

protected:
	CATSBTTickSlice() :m_ptrBegin(NULL), m_uCount(0) {}	

public:
	static inline CATSBTTickSlice* create(const char* code, CATSBTTickStruct* firstTick, uint32_t count)
	{
		if (count == 0 || firstTick == NULL)
			return NULL;

		CATSBTTickSlice* slice = new CATSBTTickSlice();
		strcpy(slice->m_strCode, code);
		slice->m_ptrBegin = firstTick;
		slice->m_uCount = count;

		return slice;
	}

	inline uint32_t size() const { return m_uCount; }

	inline bool empty() const { return (m_uCount == 0) || (m_ptrBegin == NULL); }

	inline const CATSBTTickStruct* at(int32_t idx)
	{
		if (m_ptrBegin == NULL)
			return NULL;
		return m_ptrBegin + idx;
	}
};


NS_OTP_END;

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif


#pragma pack(pop)
