﻿using System;
using System.Collections.Generic;
using CATSBTCSharp;

namespace CATSBTCSharpNetCoreDemo
{
    //策略需要继承CATSStrategy类。CATSStrategy类提供下单、撤单、行情更新、订单更新回调、查询数据、日志等各种回测相关方法。
    class StrategyDemo : CATSStrategy
    {
        string acctType = "S0"; //资金账号类型，回测可以不用，实盘时必须要
        string acct = "bt00000"; //资金账号，回测可以不用，实盘时必须要

        float lastPrice = 0;
        Dictionary<String, uint> ordersUnFinish = new Dictionary<string, uint>();
        static object ordersUnFinishLock = new object();

        int orderNum = 0;

        string submitReqNo = "0";
        string cancelReqNo = "0";
        bool submitOrderFinished = false;
        bool loadDataTest = false;
        CommodityInfo commodity;
        public override int init(Dictionary<string, string> param)
        {
            //set_debug(true); //debug模式下日志输出更加丰富
            log_porter = true;
            foreach (KeyValuePair<string, string> kv in param)
            {
                infolog(true, "C# strategy:init:param:" + kv.Key + " = " + kv.Value);
            }

            return 0;
        }
        public override int on_init()
        {
            test_load_data();
            int ret = sub_ticks("600000.SH");
            if (ret == 0)
            {
                infolog(true, "C# strategy: sub_ticks successful:symbol=600000.SH");
            }

            ret = sub_bars("600000.SH");
            if (ret == 0)
            {
                infolog(true, "C# strategy: sub_bars successful:symbol=600000.SH");
            }

            //如果使用了acctType和acct，需要显示调用init_fund和init_position
            //init_fund(1000000, acctType, acct);
            init_fund(1000000, 0, 0, 0, 0, 0.001, acctType, acct);
            //init_position("600000.SH", 100000, 24, acctType, acct);

            /*
            ret = sub_transaction("600000.SH");
            if (ret == 0)
            {
                infolog(true, "C# strategy: sub_transaction successful:symbol=600000.SH");
            }
            */
            commodity = new CommodityInfo();
            ret = get_commodity_info("600000.SH", out commodity);
            infolog(true, "C# strategy: on_init: get_commodity_info:lotsize=" + commodity.lotsize + ", ticksize=" + commodity.ticksize + ", minorderqty=" + commodity.minorderqty + ", multiplier=" + commodity.multiplier + ", commodity_type=" + commodity.commodity_type);
            return 0;
        }

        public override void on_cancel(string orderNo, bool bSuccess, string message, dynamic arg)
        {
            string str = arg;
            infolog(true, "C# strategy: on_cancel:orderNo=" + orderNo + ", bSuccess=" + bSuccess + ", message=" + message + ", arg=" + str);
        }

        public override void on_entrust(string orderNo, bool bSuccess, string message, dynamic arg)
        {
            CommodityInfo commodity = arg;
            infolog(true, "C# strategy: on_entrust:orderNo=" + orderNo + ", bSuccess=" + bSuccess + ", message=" + message + ", arg=" + commodity.ticksize);
            if (bSuccess)
            {
                uint now = get_HHMMSSsss();
                lock (ordersUnFinishLock)
                {
                    ordersUnFinish.Add(orderNo, now);
                }
            }
        }

        public override void on_tick(TickStruct data)
        {
            //infolog(true, "C# strategy: on_tick:" + data.toString());

            //if (submitOrderFinished) return;

            OrderRequest req = new OrderRequest();
            req.acctType = acctType;
            req.acct = acct;
            req.symbol = data.code;
            req.qty = 100;
            req.priceType = "0";

            if (data.action_time < 930000)
            {
                return;
            }
            orderNum++;
            if (orderNum % 5 != 0)
            {
                return;
            }

            if (data.price == lastPrice)
            {
                return;
            }
            else if (data.price < lastPrice)
            {
                req.tradeSide = "1";
                req.price = (float)(data.price + 0.01);
            }
            else if (data.price > lastPrice)
            {
                req.tradeSide = "2";
                req.price = (float)(data.price - 0.01);
            }

            submitReqNo = "reqNo:" + orderNum.ToString();
            int ret = submit_order(req, commodity);
            if (ret == 0)
            {
                infolog(true, "C# strategy: submit_order success.");
                submitOrderFinished = true;
            }
            else
            {
                infolog(true, "C# strategy: submit_order failed.");
            }

            lastPrice = (float)data.price;
            uint time = get_HHMMSSsss();


            lock (ordersUnFinishLock)
            {
                foreach (var item in ordersUnFinish)
                {
                    //5秒
                    if (item.Value - time > 5000)
                    {
                        cancelReqNo = "cancelReqNo:" + orderNum.ToString();
                        ret = cancel_order(item.Key, acctType, acct, submitReqNo);
                        if (ret == 0)
                        {
                            infolog(true, "C# strategy: cancel_order succ.");
                        }
                        else
                        {
                            infolog(true, "C# strategy: cancel_order failed.");
                        }
                    }
                }
            }
            if (!loadDataTest)
            {
                //test_load_data();
            }
            loadDataTest = true;

        }

        public override void on_bar(BarStruct data)
        {
            //infolog(true, "C# strategy: on_tick:" + data.toString());

            //if (submitOrderFinished) return;

            OrderRequest req = new OrderRequest();
            req.acctType = acctType;
            req.acct = acct;
            req.symbol = data.code;
            req.qty = 100;
            req.priceType = "0";

            if (data.time < 930000)
            {
                return;
            }
            orderNum++;
            if (orderNum % 5 != 0)
            {
                return;
            }

            if (data.close == lastPrice)
            {
                return;
            }
            else if (data.close < lastPrice)
            {
                req.tradeSide = "1";
                req.price = (float)(data.close + 0.01);
            }
            else if (data.close > lastPrice)
            {
                req.tradeSide = "2";
                req.price = (float)(data.close - 0.01);
            }

            submitReqNo = "reqNo:" + orderNum.ToString();
            int ret = submit_order(req, commodity);
            if (ret == 0)
            {
                infolog(true, "C# strategy: submit_order success.");
                submitOrderFinished = true;
            }
            else
            {
                infolog(true, "C# strategy: submit_order failed.");
            }

            lastPrice = (float)data.close;
            uint time = get_HHMMSSsss();

            lock (ordersUnFinishLock)
            {
                foreach (var item in ordersUnFinish)
                {
                    //5秒
                    if (item.Value - time > 5000)
                    {
                        cancelReqNo = "cancelReqNo:" + orderNum.ToString();
                        ret = cancel_order(item.Key, acctType, acct, submitReqNo);
                        if (ret == 0)
                        {
                            infolog(true, "C# strategy: cancel_order succ.");
                        }
                        else
                        {
                            infolog(true, "C# strategy: cancel_order failed.");
                        }
                    }
                }
            }
            if (!loadDataTest)
            {
                //test_load_data();
            }
            loadDataTest = true;

        }

        public override void on_transaction(TransStruct data)
        {
            infolog(true, "C# strategy: on_transaction:data=" + data.toString());
        }

        public override void on_order(OrderUpdateData data)
        {
            infolog(true, "C# strategy: on_order:symbol=" + data.symbol + ", orderNo=" + data.orderNo);
            if (data.status != OrderUpdateStatus.NEW && data.status != OrderUpdateStatus.PARTIAL_FILLED)
            {
                lock (ordersUnFinishLock)
                {
                    if (ordersUnFinish.ContainsKey(data.orderNo))
                    {
                        ordersUnFinish.Remove(data.orderNo);
                    }
                }
            }
        }

        public override void on_trade(TradeData data)
        {
            infolog(true, "C# strategy: on_trade:symbol=" + data.symbol + ", orderNo=" + data.orderNo);
        }

        public override int on_session_begin(int date)
        {
            orderNum = 0;
            loadDataTest = false;
            return 0;
        }

        public override int on_session_end(int date)
        {
            if (ordersUnFinish.Count > 0)
            {
                warnlog(true, "C# strategy: on_session_end:unfinished orders: size=" + ordersUnFinish.Count);
                foreach (string key in ordersUnFinish.Keys)
                {
                    warnlog(true, "C# strategy: on_session_end:unfinished orders: orderNo=" + key + ", time=" + ordersUnFinish[key]);
                }
            }

            ordersUnFinish.Clear();
            return 0;
        }

        void test_get_fund()
        {
            Fund fund = new Fund();
            int yet = get_fund(out fund);
            if (yet == 0)
            {
                infolog(true, "C# strategy: fund:currentBalance=" + fund.currentBalance + ", enabledBalance=" + fund.enabledBalance);
            }
        }

        void test_get_position()
        {
            Dictionary<string, Position> positions = new Dictionary<string, Position>();
            int yet = get_positions(out positions);
            foreach (Position pos in positions.Values)
            {
                infolog(true, "C# strategy: positions:symbol=" + pos.symbol + ", currentQty=" + pos.currentQty);
            }
        }

        void test_get_tick_slice(int date)
        {
            List<TickStruct> ticks = get_tick_slice("600000.SH", date, 91500000, date, 150000000);
            if (ticks != null)
            {
                foreach (TickStruct tick in ticks)
                {
                    infolog(true, "C# strategy: on_session_begin:tick=" + tick.toString());
                }
            }
        }

        void test_load_data()
        {
            DataTable table;
            int ret = load_table("AShareEODPrices", "600000.SH", "2021-07-01", "2021-07-10", "", "", out table);
            if (ret == 0)
            {
                for (int i = 0; i < table.rows.Count; i++)
                {
                    DataRow row = table.rows[i];
                    string rowStr = String.Empty;
                    for (int j = 0; j < row.fields.Count; j++)
                    {
                        rowStr += row.fields[j].name;
                        rowStr += "=";
                        rowStr += row.fields[j].value;
                        if (j < row.fields.Count - 1)
                        {
                            rowStr += ",";
                        }
                    }
                    infolog(true, $"C# strategy: load_table:row{i}: {rowStr}");
                }
            }
            else
            {
                infolog(true, $"C# strategy: load_table:failed, ret={ret}");
            }
        }
    }
}
