# -*- coding:utf-8 -*-

from ICatsBtPythonStrategy import ICatsBtPythonStrategy
from CatsBtPythonStrategy import BarStruct, BarStructC
import copy


# 回测V2实盘没有直接支持分钟线策略，可以考虑通过快照行情生成分钟线
# 策略启动后才开始生成，故不支持获取当天历史分钟线数据
# 未考虑行情中断或不活跃的影响
class BarSimulateStrategy(ICatsBtPythonStrategy):
    acct_type = 'S0'  # 资金账户类型
    acct = 'bt00000'  # 资金账户
    bar_start_hour = 0  # Bar开始时间，小时
    bar_start_minute = 0  # Bar开始时间，分钟
    bar_start_second = 0  # Bar开始时间，秒
    bar_start_turnover = 0 # Bar开始成交额
    curr_time = 0  # 当前时间
    pre_close = 0  # Bar前收价
    curr_bar = None  # 当前Bar
    bar_list = []  # 当天历史Bar，程序启动后

    # 回测系统初始化回调
    def on_init(self):
        super().sub_ticks("600030.SH")  # 订阅快照行情
        super().sub_order(self.acct_type, self.acct)  # 订阅订单更新推送
        return 0

    # 日初回调
    def on_session_begin(self, date):
        return 0

    # 快照行情推送回调
    def on_tick(self, data):
        # 跳过盘前集中竞价和尾盘集中竞价
        action_time_s = data.action_time // 1000
        if action_time_s < 93000:
            return
        if action_time_s >= 145700:
            if self.curr_bar is not None:
                self.close_bar()
            return

        if self.curr_bar is None:
            self.create_bar(data)
        else:
            curr_hour = action_time_s // 10000
            curr_minute = action_time_s % 10000 // 100
            curr_second = action_time_s % 100
            if curr_hour >= self.bar_start_hour:
                if curr_minute == self.bar_start_minute:
                    self.update_bar(data)
                    if curr_second > 57:
                        self.close_bar()
                elif curr_minute > self.bar_start_minute:
                    self.close_bar()
                    if action_time_s < 145700:
                        self.create_bar(data)

    def on_bar(self, data):
        super().infolog_c(True, 'on_bar:' + data.toString())

    def create_bar(self, data):
        bar_c = BarStructC()
        self.curr_bar = BarStruct(bar_c)
        self.curr_bar.code = data.code
        self.curr_bar.open = data.price
        self.curr_bar.high = data.price
        self.curr_bar.low = data.price
        self.curr_bar.close = data.price
        self.curr_bar.pre_close = self.pre_close
        self.curr_bar.turn_over = 0
        self.curr_bar.volume = data.volume
        self.curr_bar.date = data.trading_date

        action_time_s = data.action_time // 1000
        self.curr_bar.time = action_time_s // 100
        self.bar_start_hour = action_time_s // 10000
        self.bar_start_minute = action_time_s % 10000 // 100
        self.bar_start_second = action_time_s % 100

    def update_bar(self, data):
        if self.curr_bar.high < data.price:
            self.curr_bar.high = data.price
        if self.curr_bar.low > data.price:
            self.curr_bar.low = data.price
        self.curr_bar.close = data.price
        self.curr_bar.turn_over = data.total_turnover
        self.curr_bar.volume += data.volume

    def close_bar(self):
        self.pre_close = self.curr_bar.close
        if len(self.bar_list) == 0:     # 第1个Bar
            self.bar_start_turnover = self.curr_bar.turn_over
            self.curr_bar.turn_over = 0
        else:
            self.curr_bar.turn_over = self.curr_bar.turn_over - self.bar_start_turnover
            self.bar_start_turnover += self.curr_bar.turn_over
        self.on_bar(self.curr_bar)
        self.bar_list.append(copy.deepcopy(self.curr_bar))
        self.curr_bar = None
