# -*- coding:utf-8 -*-

from ICatsBtPythonStrategy import ICatsBtPythonStrategy
from CatsBtPythonStrategy import OrderRequest


# 策略逻辑：行情触发，上涨0.5个点时买入
class SimpleDemo(ICatsBtPythonStrategy):
    acct_type = 'S0'            # 资金账户类型
    acct = 'bt00000'            # 资金账户
    place_order_done = False    # 控制下单次数，每天只下1次

    # 回测系统初始化回调
    def on_init(self):
        super().sub_ticks("600030.SH")      # 订阅快照行情
        return 0

    # 日初回调
    def on_session_begin(self, date):
        self.place_order_done = False
        return 0

    def on_session_end(self, date):
        return 0

    # 快照行情推送回调
    def on_tick(self, data):
        # 盘前集中竞价不交易
        current = super().get_HHMMSSsss() / 1000
        if current < 93000:
            return

        if not self.place_order_done and data.price >= data.pre_close * 1.005:
            req = OrderRequest()
            req.acctType = self.acct_type
            req.acct = self.acct
            req.symbol = data.code
            req.qty = 1000
            req.priceType = '0'     # 限价
            req.tradeSide = '1'     # 买入
            req.price = data.price

            ret = super().submit_order(req)
            if ret == 0:
                self.place_order_done = True
                super().infolog_c(True, 'submit order success')
            else:
                super().infolog_c(True, 'submit order failed, ret={}'.format(ret))
